package DAO;

import Exceptions.IngredientDAOException;
import POJO.Ingredient;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class IngredientDAO {
    // JDBC URL, username, and password of PostgresSQL server
    private static final String url = "jdbc:postgresql://localhost:5432/pizzeria";
    private static final String user = "gwendalmargelyetu";
    private static final String password = "moi";

    // JDBC variables for opening, closing and managing connection
    private static Connection connection;
    private static PreparedStatement preparedStatement;
    private static ResultSet resultSet;

    // SQL queries
    private static final String SELECT_ALL_QUERY = "SELECT * FROM ingredients";
    private static final String INSERT_QUERY = "INSERT INTO ingredients(nom, price) VALUES (?, ?)";
    private static final String SELECT_BY_ID_QUERY = "SELECT nom,price FROM ingredients WHERE id = ?";
    private static final String DELETE_QUERY = "DELETE FROM ingredients WHERE id = ?";
    private static final String UPDATE_QUERY = "UPDATE ingredients SET nom = ?, price = ? WHERE id = ?";

    // Method to establish database connection
    static Connection connect() throws IngredientDAOException {
        try {
            connection = DriverManager.getConnection(url, user, password);
        } catch (SQLException e) {
            throw new IngredientDAOException("Error establishing database connection", e);
        }
        return connection;
    }

    // Method to close database connection
    public static void close() throws IngredientDAOException {
        try {
            if (resultSet != null) {
                resultSet.close();
            }
            if (preparedStatement != null) {
                preparedStatement.close();
            }
            if (connection != null) {
                connection.close();
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error closing database resources", e);
        }
    }

    // Method to retrieve all ingredients from the database
    public List<Ingredient> getAllIngredients() throws IngredientDAOException {
        List<Ingredient> ingredients = new ArrayList<>();
        try {
            connect();
            preparedStatement = connection.prepareStatement(SELECT_ALL_QUERY);
            resultSet = preparedStatement.executeQuery();

            while (resultSet.next()) {
                int id = resultSet.getInt("id");
                String name = resultSet.getString("nom");
                double price = resultSet.getDouble("price");
                Ingredient ingredient = new Ingredient(id, name, price);
                ingredients.add(ingredient);
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error fetching ingredients", e);
        } finally {
            close();
        }
        return ingredients;
    }

    // Method to insert a new ingredient into the database
    public static Ingredient insertIngredient(Ingredient ingredient) throws IngredientDAOException {
        try {
            connect();
            preparedStatement = connection.prepareStatement(INSERT_QUERY);
            preparedStatement.setString(1, ingredient.getName());
            preparedStatement.setDouble(2, ingredient.getPrice());
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new IngredientDAOException("Error inserting ingredient", e);
        } finally {
            close();
        }
        return ingredient;
    }

    // Method to retrieve an ingredient by its ID
    public static Ingredient getIngredient(int id) throws IngredientDAOException {
        Ingredient ingredient = null;
        try {
            connect();
            preparedStatement = connection.prepareStatement(SELECT_BY_ID_QUERY);
            preparedStatement.setInt(1, id);
            resultSet = preparedStatement.executeQuery();

            if (resultSet.next()) {
                String name = resultSet.getString("nom");
                double price = resultSet.getDouble("price");
                ingredient = new Ingredient(id, name, price);
            }
        } catch (SQLException e) {
            throw new IngredientDAOException("Error fetching ingredient", e);
        } finally {
            close();
        }
        return ingredient;
    }

    public void deleteIngredient(int id) throws IngredientDAOException {
        try {
            connect();
            preparedStatement = connection.prepareStatement(DELETE_QUERY);
            preparedStatement.setInt(1, id);
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new IngredientDAOException("Error deleting ingredient", e);
        } finally {
            close();
        }
    }

    public static void updateIngredient(Ingredient ingredient) throws IngredientDAOException {
        try {
            connect();
            preparedStatement = connection.prepareStatement(UPDATE_QUERY);
            preparedStatement.setString(1, ingredient.getName());
            preparedStatement.setDouble(2, ingredient.getPrice());
            preparedStatement.setInt(3, ingredient.getId());
            preparedStatement.executeUpdate();
        } catch (SQLException e) {
            throw new IngredientDAOException("Error updating ingredient", e);
        } finally {
            close();
        }
    }
}
